@@ -10,6 +10,20 @@
 #include "png_vf.h"
 
 
+/**
+ * @defgroup snapshot snapshot
+ *
+ * Take snapshot of the video stream and save it as PNG-files
+ *
+ *
+ * Commands:
+ *
+ \verbatim
+ O       Take video snapshot
+ \endverbatim
+ */
+
+
 static bool flag_enc, flag_dec;
 
 
