@@ -3,6 +3,7 @@
  *
  * Copyright (C) 2010 Creytiv.com
  */
+#define _DEFAULT_SOURCE 1
 #define _BSD_SOURCE 1
 #include <unistd.h>
 #include <stdlib.h>
@@ -36,6 +37,13 @@
 #endif
 
 
+/**
+ * @defgroup v4l2 v4l2
+ *
+ * V4L2 (Video for Linux 2) video-source module
+ */
+
+
 struct buffer {
 	void  *start;
 	size_t length;
