@@ -9,6 +9,17 @@
 #include <baresip.h>
 
 
+/**
+ * @defgroup vumeter vumeter
+ *
+ * Simple ASCII VU-meter for the audio-signal.
+ *
+ * The Volume unit (VU) meter module takes the audio-signal as input
+ * and prints a simple ASCII-art bar for the recording and playback levels.
+ * It is using the aufilt API to get the audio samples.
+ */
+
+
 struct vumeter_enc {
 	struct aufilt_enc_st af;  /* inheritance */
 	struct tmr tmr;
@@ -157,7 +168,7 @@ static int decode_update(struct aufilt_dec_st **stp, void **ctx,
 
 static int encode(struct aufilt_enc_st *st, int16_t *sampv, size_t *sampc)
 {
-	struct vumeter_enc *vu = (struct vumeter_enc *)st;
+	struct vumeter_enc *vu = (void *)st;
 
 	vu->avg_rec = calc_avg_s16(sampv, *sampc);
 	vu->started = true;
@@ -168,7 +179,7 @@ static int encode(struct aufilt_enc_st *st, int16_t *sampv, size_t *sampc)
 
 static int decode(struct aufilt_dec_st *st, int16_t *sampv, size_t *sampc)
 {
-	struct vumeter_dec *vu = (struct vumeter_dec *)st;
+	struct vumeter_dec *vu = (void *)st;
 
 	vu->avg_play = calc_avg_s16(sampv, *sampc);
 	vu->started = true;
