@@ -3,6 +3,7 @@
  *
  * Copyright (C) 2010 Creytiv.com
  */
+#define _DEFAULT_SOURCE 1
 #define _BSD_SOURCE 1
 #include <unistd.h>
 #ifndef SOLARIS
@@ -16,7 +17,12 @@
 #include <baresip.h>
 
 
-/*
+/**
+ * @defgroup x11grab x11grab
+ *
+ * X11 window-grabbing video-source module
+ *
+ *
  * XXX: add option to select a specific X window and x,y offset
  */
 
