@@ -53,7 +53,7 @@
  */
 
 enum {
-	AUDIO_SAMPSZ    = 1920,
+	AUDIO_SAMPSZ    = 3*1920  /* Max samples, 48000Hz 2ch at 60ms */
 };
 
 
@@ -91,7 +91,6 @@ struct autx {
 	uint32_t ts_tel;              /**< Timestamp for Telephony Events  */
 	size_t psize;                 /**< Packet size for sending         */
 	bool marker;                  /**< Marker bit for outgoing RTP     */
-	bool is_g722;                 /**< Set if encoder is G.722 codec   */
 	bool muted;                   /**< Audio source is muted           */
 	int cur_key;                  /**< Currently transmitted event     */
 
@@ -240,14 +239,26 @@ static inline uint32_t calc_nsamp(uint32_t srate, uint8_t channels,
 
 
 /**
- * Get the DSP samplerate for an audio-codec (exception for G.722)
+ * Get the DSP samplerate for an audio-codec (exception for G.722 and MPA)
  */
 static inline uint32_t get_srate(const struct aucodec *ac)
 {
 	if (!ac)
 		return 0;
 
-	return !str_casecmp(ac->name, "G722") ? 16000 : ac->srate;
+	return ac->srate;
+}
+
+
+/**
+ * Get the DSP channels for an audio-codec (exception for MPA)
+ */
+static inline uint32_t get_ch(const struct aucodec *ac)
+{
+	if (!ac)
+		return 0;
+
+	return !str_casecmp(ac->name, "MPA") ? 2 : ac->ch;
 }
 
 
@@ -257,7 +268,7 @@ static inline uint32_t get_framesize(const struct aucodec *ac,
 	if (!ac)
 		return 0;
 
-	return calc_nsamp(get_srate(ac), ac->ch, ptime);
+	return calc_nsamp(get_srate(ac), get_ch(ac), ptime);
 }
 
 
@@ -266,7 +277,7 @@ static bool aucodec_equal(const struct aucodec *a, const struct aucodec *b)
 	if (!a || !b)
 		return false;
 
-	return get_srate(a) == get_srate(b) && a->ch == b->ch;
+	return get_srate(a) == get_srate(b) && get_ch(a) == get_ch(b);
 }
 
 
@@ -279,13 +290,18 @@ static int add_audio_codec(struct audio *a, struct sdp_media *m,
 		return 0;
 	}
 
-	if (!in_range(&a->cfg.channels, ac->ch)) {
+	if (!in_range(&a->cfg.channels, get_ch(ac))) {
 		debug("audio: skip codec with %uch (audio range %uch-%uch)\n",
-		      ac->ch, a->cfg.channels.min, a->cfg.channels.max);
+		      get_ch(ac), a->cfg.channels.min, a->cfg.channels.max);
 		return 0;
 	}
 
-	return sdp_format_add(NULL, m, false, ac->pt, ac->name, ac->srate,
+	if (ac->crate < 8000) {
+		warning("audio: illegal clock rate %u\n", ac->crate);
+		return EINVAL;
+	}
+
+	return sdp_format_add(NULL, m, false, ac->pt, ac->name, ac->crate,
 			      ac->ch, ac->fmtp_ench, ac->fmtp_cmph, ac, false,
 			      "%s", ac->fmtp);
 }
@@ -305,6 +321,7 @@ static void encode_rtp_send(struct audio *a, struct autx *tx,
 			    int16_t *sampv, size_t sampc)
 {
 	size_t frame_size;  /* number of samples per channel */
+	size_t sampc_rtp;
 	size_t len;
 	int err;
 
@@ -315,7 +332,12 @@ static void encode_rtp_send(struct audio *a, struct autx *tx,
 	len = mbuf_get_space(tx->mb);
 
 	err = tx->ac->ench(tx->enc, mbuf_buf(tx->mb), &len, sampv, sampc);
-	if (err) {
+	if ((err & 0xffff0000) == 0x00010000) {
+		/* MPA needs some special treatment here */
+		tx->ts = err & 0xffff;
+		err = 0;
+	}
+	else if (err) {
 		warning("audio: %s encode error: %d samples (%m)\n",
 			tx->ac->name, sampc, err);
 		goto out;
@@ -325,16 +347,23 @@ static void encode_rtp_send(struct audio *a, struct autx *tx,
 	tx->mb->end = STREAM_PRESZ + len;
 
 	if (mbuf_get_left(tx->mb)) {
-
-		err = stream_send(a->strm, tx->marker, -1, tx->ts, tx->mb);
-		if (err)
-			goto out;
+		if (len) {
+			err = stream_send(a->strm, tx->marker, -1,
+					tx->ts, tx->mb);
+			if (err)
+				goto out;
+		}
 	}
 
+	/* Convert from audio samplerate to RTP clockrate */
+	sampc_rtp = sampc * tx->ac->crate / tx->ac->srate;
+
 	/* The RTP clock rate used for generating the RTP timestamp is
 	 * independent of the number of channels and the encoding
+	 * However, MPA support variable packet durations. Thus, MPA
+	 * should update the ts according to its current internal state.
 	 */
-	frame_size = (tx->is_g722 ? sampc/2 : sampc) / tx->ac->ch;
+	frame_size = sampc_rtp / get_ch(tx->ac);
 
 	tx->ts += (uint32_t)frame_size;
 
@@ -535,6 +564,8 @@ static int aurx_stream_decode(struct aurx *rx, struct mbuf *mb)
 				   mbuf_buf(mb), mbuf_get_left(mb));
 	}
 	else if (rx->ac->plch) {
+		sampc = rx->ac->srate * rx->ac->ch * rx->ptime / 1000;
+
 		err = rx->ac->plch(rx->dec, rx->sampv, &sampc);
 	}
 	else {
@@ -793,7 +824,7 @@ static void aufilt_param_set(struct aufilt_prm *prm,
 	}
 
 	prm->srate      = get_srate(ac);
-	prm->ch         = ac->ch;
+	prm->ch         = get_ch(ac);
 	prm->ptime      = ptime;
 }
 
@@ -922,7 +953,7 @@ static int start_player(struct aurx *rx, struct audio *a)
 	if (!ac)
 		return 0;
 
-	channels_dsp = ac->ch;
+	channels_dsp = get_ch(ac);
 
 	if (a->cfg.srate_play && a->cfg.srate_play != srate_dsp) {
 		resamp = true;
@@ -938,14 +969,14 @@ static int start_player(struct aurx *rx, struct audio *a)
 
 		info("audio: enable auplay resampler:"
 		     " %uHz/%uch --> %uHz/%uch\n",
-		     get_srate(ac), ac->ch, srate_dsp, channels_dsp);
+		     get_srate(ac), get_ch(ac), srate_dsp, channels_dsp);
 
 		rx->sampv_rs = mem_zalloc(AUDIO_SAMPSZ * 2, NULL);
 		if (!rx->sampv_rs)
 			return ENOMEM;
 
 		err = auresamp_setup(&rx->resamp,
-				     get_srate(ac), ac->ch,
+				     get_srate(ac), get_ch(ac),
 				     srate_dsp, channels_dsp);
 		if (err) {
 			warning("audio: could not setup auplay resampler"
@@ -1000,7 +1031,7 @@ static int start_source(struct autx *tx, struct audio *a)
 	if (!ac)
 		return 0;
 
-	channels_dsp = ac->ch;
+	channels_dsp = get_ch(ac);
 
 	if (a->cfg.srate_src && a->cfg.srate_src != srate_dsp) {
 		resamp = true;
@@ -1016,7 +1047,7 @@ static int start_source(struct autx *tx, struct audio *a)
 
 		info("audio: enable ausrc resampler:"
 		     " %uHz/%uch <-- %uHz/%uch\n",
-		     get_srate(ac), ac->ch, srate_dsp, channels_dsp);
+		     get_srate(ac), get_ch(ac), srate_dsp, channels_dsp);
 
 		tx->sampv_rs = mem_zalloc(AUDIO_SAMPSZ * 2, NULL);
 		if (!tx->sampv_rs)
@@ -1024,7 +1055,7 @@ static int start_source(struct autx *tx, struct audio *a)
 
 		err = auresamp_setup(&tx->resamp,
 				     srate_dsp, channels_dsp,
-				     get_srate(ac), ac->ch);
+				     get_srate(ac), get_ch(ac));
 		if (err) {
 			warning("audio: could not setup ausrc resampler"
 				" (%m)\n", err);
@@ -1169,14 +1200,13 @@ int audio_encoder_set(struct audio *a, const struct aucodec *ac,
 
 	if (ac != tx->ac) {
 		info("audio: Set audio encoder: %s %uHz %dch\n",
-		     ac->name, get_srate(ac), ac->ch);
+		     ac->name, get_srate(ac), get_ch(ac));
 
 		/* Audio source must be stopped first */
 		if (reset) {
 			tx->ausrc = mem_deref(tx->ausrc);
 		}
 
-		tx->is_g722 = (0 == str_casecmp(ac->name, "G722"));
 		tx->enc = mem_deref(tx->enc);
 		tx->ac = ac;
 	}
@@ -1193,7 +1223,7 @@ int audio_encoder_set(struct audio *a, const struct aucodec *ac,
 		}
 	}
 
-	stream_set_srate(a->strm, get_srate(ac), get_srate(ac));
+	stream_set_srate(a->strm, ac->crate, ac->crate);
 	stream_update_encoder(a->strm, pt_tx);
 
 	if (!tx->ausrc) {
@@ -1221,7 +1251,7 @@ int audio_decoder_set(struct audio *a, const struct aucodec *ac,
 	if (ac != rx->ac) {
 
 		info("audio: Set audio decoder: %s %uHz %dch\n",
-		     ac->name, get_srate(ac), ac->ch);
+		     ac->name, get_srate(ac), get_ch(ac));
 
 		rx->pt = pt_rx;
 		rx->ac = ac;
@@ -1236,7 +1266,7 @@ int audio_decoder_set(struct audio *a, const struct aucodec *ac,
 		}
 	}
 
-	stream_set_srate(a->strm, get_srate(ac), get_srate(ac));
+	stream_set_srate(a->strm, ac->crate, ac->crate);
 
 	if (reset) {
 
@@ -1370,7 +1400,8 @@ static int aucodec_print(struct re_printf *pf, const struct aucodec *ac)
 	if (!ac)
 		return 0;
 
-	return re_hprintf(pf, "%s %uHz/%dch", ac->name, get_srate(ac), ac->ch);
+	return re_hprintf(pf, "%s %uHz/%dch",
+			  ac->name, get_srate(ac), get_ch(ac));
 }
 
 
