@@ -24,7 +24,6 @@
 /** Call constants */
 enum {
 	PTIME           = 20,    /**< Packet time for audio               */
-	LOCAL_TIMEOUT   = 120,   /**< Incoming call timeout in [seconds]  */
 };
 
 
@@ -78,6 +77,9 @@ struct call {
 	void *arg;                /**< Handler argument                     */
 
 	struct config_avt config_avt;
+	struct config_call config_call;
+
+	uint32_t rtp_timeout_ms;  /**< RTP Timeout in [ms]                  */
 };
 
 
@@ -219,7 +221,7 @@ static void invite_timeout(void *arg)
 	struct call *call = arg;
 
 	info("%s: Local timeout after %u seconds\n",
-	     call->peer_uri, LOCAL_TIMEOUT);
+	     call->peer_uri, call->config_call.local_timeout);
 
 	call_event_handler(call, CALL_EVENT_CLOSED, "Local timeout");
 }
@@ -420,6 +422,22 @@ static void menc_error_handler(int err, void *arg)
 }
 
 
+static void stream_error_handler(struct stream *strm, int err, void *arg)
+{
+	struct call *call = arg;
+	MAGIC_CHECK(call);
+
+	info("call: error in \"%s\" rtp stream (%m)\n",
+		sdp_media_name(stream_sdpmedia(strm)), err);
+
+	call->scode = 701;
+	set_state(call, STATE_TERMINATED);
+
+	call_stream_stop(call);
+	call_event_handler(call, CALL_EVENT_CLOSED, "rtp stream error");
+}
+
+
 /**
  * Allocate a new Call state object
  *
@@ -445,6 +463,7 @@ int call_alloc(struct call **callp, const struct config *cfg, struct list *lst,
 	       call_event_h *eh, void *arg)
 {
 	struct call *call;
+	struct le *le;
 	enum vidmode vidmode = prm ? prm->vidmode : VIDMODE_OFF;
 	bool use_video = true, got_offer = false;
 	int label = 0;
@@ -460,6 +479,7 @@ int call_alloc(struct call **callp, const struct config *cfg, struct list *lst,
 	MAGIC_INIT(call);
 
 	call->config_avt = cfg->avt;
+	call->config_call = cfg->call;
 
 	tmr_init(&call->tmr_inv);
 
@@ -477,7 +497,8 @@ int call_alloc(struct call **callp, const struct config *cfg, struct list *lst,
 		goto out;
 
 	/* Init SDP info */
-	err = sdp_session_alloc(&call->sdp, net_laddr_af(call->af));
+	err = sdp_session_alloc(&call->sdp,
+				net_laddr_af(baresip_network(), call->af));
 	if (err)
 		goto out;
 
@@ -492,7 +513,8 @@ int call_alloc(struct call **callp, const struct config *cfg, struct list *lst,
 
 	/* Initialise media NAT handling */
 	if (acc->mnat) {
-		err = acc->mnat->sessh(&call->mnats, net_dnsc(), call->af,
+		err = acc->mnat->sessh(&call->mnats,
+				       net_dnsc(baresip_network()), call->af,
 				       acc->stun_host, acc->stun_port,
 				       acc->stun_user, acc->stun_pass,
 				       call->sdp, !got_offer,
@@ -564,6 +586,15 @@ int call_alloc(struct call **callp, const struct config *cfg, struct list *lst,
 		call->not = mem_ref(xcall->not);
 	}
 
+	FOREACH_STREAM {
+		struct stream *strm = le->data;
+		stream_set_error_handler(strm, stream_error_handler, call);
+	}
+
+	if (cfg->avt.rtp_timeout) {
+		call_enable_rtp_timeout(call, cfg->avt.rtp_timeout*1000);
+	}
+
 	list_append(lst, &call->le, call);
 
  out:
@@ -1030,6 +1061,16 @@ static void sipsess_estab_handler(const struct sip_msg *msg, void *arg)
 
 	call_stream_start(call, true);
 
+	if (call->rtp_timeout_ms) {
+
+		struct le *le;
+
+		FOREACH_STREAM {
+			struct stream *strm = le->data;
+			stream_enable_rtp_timeout(strm, call->rtp_timeout_ms);
+		}
+	}
+
 	/* the transferor will hangup this call */
 	if (call->not) {
 		(void)call_notify_sipfrag(call, 200, "OK");
@@ -1311,7 +1352,10 @@ int call_accept(struct call *call, struct sipsess_sock *sess_sock,
 	set_state(call, STATE_INCOMING);
 
 	/* New call */
-	tmr_start(&call->tmr_inv, LOCAL_TIMEOUT*1000, invite_timeout, call);
+	if (call->config_call.local_timeout) {
+		tmr_start(&call->tmr_inv, call->config_call.local_timeout*1000,
+			  invite_timeout, call);
+	}
 
 	if (!call->acc->mnat)
 		call_event_handler(call, CALL_EVENT_INCOMING, call->peer_uri);
@@ -1498,7 +1542,8 @@ int call_reset_transp(struct call *call)
 	if (!call)
 		return EINVAL;
 
-	sdp_session_set_laddr(call->sdp, net_laddr_af(call->af));
+	sdp_session_set_laddr(call->sdp,
+			      net_laddr_af(baresip_network(), call->af));
 
 	return call_modify(call);
 }
@@ -1709,3 +1754,12 @@ bool call_is_outgoing(const struct call *call)
 {
 	return call ? call->outgoing : false;
 }
+
+
+void call_enable_rtp_timeout(struct call *call, uint32_t timeout_ms)
+{
+	if (!call)
+		return;
+
+	call->rtp_timeout_ms = timeout_ms;
+}
