@@ -15,8 +15,11 @@
 #define MOD_PRE ""  /**< Module prefix */
 
 
+#undef SA_INIT
+#define SA_INIT { { {0} }, 0}
+
+
 /** Core Run-time Configuration - populated from config file */
-/** @todo: move config parsing/decoding to a module */
 static struct config core_config = {
 
 	/** SIP User-Agent */
@@ -27,8 +30,14 @@ static struct config core_config = {
 		""
 	},
 
+	/** Call config */
+	{
+		120
+	},
+
 	/** Audio */
 	{
+		"",
 		"","",
 		"","",
 		"","",
@@ -61,12 +70,15 @@ static struct config core_config = {
 		true,
 		false,
 		{5, 10},
-		false
+		false,
+		0
 	},
 
 	/* Network */
 	{
-		""
+		"",
+		{ {""} },
+		0
 	},
 
 #ifdef USE_VIDEO
@@ -89,23 +101,28 @@ static int range_print(struct re_printf *pf, const struct range *rng)
 
 static int dns_server_handler(const struct pl *pl, void *arg)
 {
-	struct sa sa;
+	struct config_net *cfg = arg;
+	const size_t max_count = ARRAY_SIZE(cfg->nsv);
 	int err;
 
-	(void)arg;
+	if (cfg->nsc >= max_count) {
+		warning("config: too many DNS nameservers (max %zu)\n",
+			max_count);
+		return EOVERFLOW;
+	}
 
-	err = sa_decode(&sa, pl->p, pl->l);
+	/* Append dns_server to the network config */
+	err = pl_strcpy(pl, cfg->nsv[cfg->nsc].addr,
+			sizeof(cfg->nsv[0].addr));
 	if (err) {
-		warning("config: dns_server: could not decode `%r'\n", pl);
+		warning("config: dns_server: could not copy string (%r)\n",
+			pl);
 		return err;
 	}
 
-	err = net_dnssrv_add(&sa);
-	if (err) {
-		warning("config: failed to add nameserver %r: %m\n", pl, err);
-	}
+	++cfg->nsc;
 
-	return err;
+	return 0;
 }
 
 
@@ -141,7 +158,13 @@ int config_parse_conf(struct config *cfg, const struct conf *conf)
 	(void)conf_get_str(conf, "sip_certificate", cfg->sip.cert,
 			   sizeof(cfg->sip.cert));
 
+	/* Call */
+	(void)conf_get_u32(conf, "call_local_timeout",
+			   &cfg->call.local_timeout);
+
 	/* Audio */
+	(void)conf_get_str(conf, "audio_path", cfg->audio.audio_path,
+			   sizeof(cfg->audio.audio_path));
 	(void)conf_get_csv(conf, "audio_player",
 			   cfg->audio.play_mod,
 			   sizeof(cfg->audio.play_mod),
@@ -201,13 +224,14 @@ int config_parse_conf(struct config *cfg, const struct conf *conf)
 	(void)conf_get_range(conf, "jitter_buffer_delay",
 			     &cfg->avt.jbuf_del);
 	(void)conf_get_bool(conf, "rtp_stats", &cfg->avt.rtp_stats);
+	(void)conf_get_u32(conf, "rtp_timeout", &cfg->avt.rtp_timeout);
 
 	if (err) {
 		warning("config: configure parse error (%m)\n", err);
 	}
 
 	/* Network */
-	(void)conf_apply(conf, "dns_server", dns_server_handler, NULL);
+	(void)conf_apply(conf, "dns_server", dns_server_handler, &cfg->net);
 	(void)conf_get_str(conf, "net_interface",
 			   cfg->net.ifname, sizeof(cfg->net.ifname));
 
@@ -235,7 +259,11 @@ int config_print(struct re_printf *pf, const struct config *cfg)
 			 "sip_listen\t\t%s\n"
 			 "sip_certificate\t%s\n"
 			 "\n"
+			 "# Call\n"
+			 "call_local_timeout\t%u\n"
+			 "\n"
 			 "# Audio\n"
+			 "audio_path\t\t%s\n"
 			 "audio_player\t\t%s,%s\n"
 			 "audio_source\t\t%s,%s\n"
 			 "audio_alert\t\t%s,%s\n"
@@ -263,6 +291,7 @@ int config_print(struct re_printf *pf, const struct config *cfg)
 			 "rtcp_mux\t\t%s\n"
 			 "jitter_buffer_delay\t%H\n"
 			 "rtp_stats\t\t%s\n"
+			 "rtp_timeout\t\t%u # in seconds\n"
 			 "\n"
 			 "# Network\n"
 			 "net_interface\t\t%s\n"
@@ -276,6 +305,9 @@ int config_print(struct re_printf *pf, const struct config *cfg)
 
 			 cfg->sip.trans_bsize, cfg->sip.local, cfg->sip.cert,
 
+			 cfg->call.local_timeout,
+
+			 cfg->audio.audio_path,
 			 cfg->audio.play_mod,  cfg->audio.play_dev,
 			 cfg->audio.src_mod,   cfg->audio.src_dev,
 			 cfg->audio.alert_mod, cfg->audio.alert_dev,
@@ -298,6 +330,7 @@ int config_print(struct re_printf *pf, const struct config *cfg)
 			 cfg->avt.rtcp_mux ? "yes" : "no",
 			 range_print, &cfg->avt.jbuf_del,
 			 cfg->avt.rtp_stats ? "yes" : "no",
+			 cfg->avt.rtp_timeout,
 
 			 cfg->net.ifname
 
@@ -313,7 +346,7 @@ int config_print(struct re_printf *pf, const struct config *cfg)
 static const char *default_audio_device(void)
 {
 #if defined (ANDROID)
-	return "opensles";
+	return "opensles,nil";
 #elif defined (DARWIN)
 	return "coreaudio,nil";
 #elif defined (FREEBSD)
@@ -389,7 +422,12 @@ static int core_config_template(struct re_printf *pf, const struct config *cfg)
 			  "sip_trans_bsize\t\t128\n"
 			  "#sip_listen\t\t0.0.0.0:5060\n"
 			  "#sip_certificate\tcert.pem\n"
-			  "\n# Audio\n"
+			  "\n"
+			  "# Call\n"
+			  "call_local_timeout\t%u\n"
+			  "\n"
+			  "# Audio\n"
+			  "#audio_path\t\t/usr/share/baresip\n"
 			  "audio_player\t\t%s\n"
 			  "audio_source\t\t%s\n"
 			  "audio_alert\t\t%s\n"
@@ -401,6 +439,7 @@ static int core_config_template(struct re_printf *pf, const struct config *cfg)
 			  "#auplay_channels\t\t0\n"
 			  ,
 			  poll_method_name(poll_method_best()),
+			  cfg->call.local_timeout,
 			  default_audio_device(),
 			  default_audio_device(),
 			  default_audio_device(),
@@ -430,6 +469,7 @@ static int core_config_template(struct re_printf *pf, const struct config *cfg)
 			  "rtcp_mux\t\tno\n"
 			  "jitter_buffer_delay\t%u-%u\t\t# frames\n"
 			  "rtp_stats\t\tno\n"
+			  "#rtp_timeout\t\t60\n"
 			  "\n# Network\n"
 			  "#dns_server\t\t10.0.0.1:53\n"
 			  "#net_interface\t\t%H\n",
@@ -602,7 +642,7 @@ int config_write_template(const char *file, const struct config *cfg)
 #else
 	(void)re_fprintf(f, "#module\t\t\t" MOD_PRE "avcodec" MOD_EXT "\n");
 #endif
-	(void)re_fprintf(f, "#module\t\t\t" MOD_PRE "vpx" MOD_EXT "\n");
+	(void)re_fprintf(f, "#module\t\t\t" MOD_PRE "vp8" MOD_EXT "\n");
 
 	(void)re_fprintf(f, "\n# Video filter Modules (in encoding order)\n");
 	(void)re_fprintf(f, "#module\t\t\t" MOD_PRE "selfview" MOD_EXT "\n");
@@ -714,6 +754,11 @@ int config_write_template(const char *file, const struct config *cfg)
 			"ice_nomination\t\tregular\t# {regular,aggressive}\n"
 			"ice_mode\t\tfull\t# {full,lite}\n");
 
+	(void)re_fprintf(f,
+			"\n# Menu\n"
+			"#redial_attempts\t\t3 # Num or <inf>\n"
+			"#redial_delay\t\t5 # Delay in seconds\n");
+
 	if (f)
 		(void)fclose(f);
 
