@@ -57,6 +57,8 @@ static struct {
 	bool use_tls;                  /**< Use TLS transport               */
 	bool prefer_ipv6;              /**< Force IPv6 transport            */
 	sip_msg_h *subh;
+	ua_exit_h *exith;              /**< UA Exit handler                 */
+	void *arg;                     /**< UA Exit handler argument        */
 	char *eprm;                    /**< Extra UA parameters             */
 #ifdef USE_TLS
 	struct tls *tls;               /**< TLS Context                     */
@@ -76,6 +78,8 @@ static struct {
 	false,
 	NULL,
 	NULL,
+	NULL,
+	NULL,
 #ifdef USE_TLS
 	NULL,
 #endif
@@ -96,9 +100,9 @@ static void exit_handler(void *arg)
 	ua_event(NULL, UA_EVENT_EXIT, NULL, NULL);
 
 	debug("ua: sip-stack exit\n");
-	module_app_unload();
 
-	re_cancel();
+	if (uag.exith)
+		uag.exith(uag.arg);
 }
 
 
@@ -149,7 +153,7 @@ int ua_register(struct ua *ua)
 	struct account *acc;
 	struct le *le;
 	struct uri uri;
-	char reg_uri[64];
+	char *reg_uri = NULL;
 	char params[256] = "";
 	unsigned i;
 	int err;
@@ -160,28 +164,36 @@ int ua_register(struct ua *ua)
 	acc = ua->acc;
 	uri = ua->acc->luri;
 	uri.user = uri.password = pl_null;
-	if (re_snprintf(reg_uri, sizeof(reg_uri), "%H", uri_encode, &uri) < 0)
-		return ENOMEM;
+
+	err = re_sdprintf(&reg_uri, "%H", uri_encode, &uri);
+	if (err)
+		goto out;
 
 	if (uag.cfg && str_isset(uag.cfg->uuid)) {
 		if (re_snprintf(params, sizeof(params),
 				";+sip.instance=\"<urn:uuid:%s>\"",
-				uag.cfg->uuid) < 0)
-			return ENOMEM;
+				uag.cfg->uuid) < 0) {
+			err = ENOMEM;
+			goto out;
+		}
 	}
 
 	if (acc->regq) {
 		if (re_snprintf(&params[strlen(params)],
 				sizeof(params) - strlen(params),
-				";q=%s", acc->regq) < 0)
-			return ENOMEM;
+				";q=%s", acc->regq) < 0) {
+			err = ENOMEM;
+			goto out;
+		}
 	}
 
 	if (acc->mnat && acc->mnat->ftag) {
 		if (re_snprintf(&params[strlen(params)],
 				sizeof(params) - strlen(params),
-				";%s", acc->mnat->ftag) < 0)
-			return ENOMEM;
+				";%s", acc->mnat->ftag) < 0) {
+			err = ENOMEM;
+			goto out;
+		}
 	}
 
 	ua_event(ua, UA_EVENT_REGISTERING, NULL, NULL);
@@ -193,11 +205,14 @@ int ua_register(struct ua *ua)
 				   acc->regint, acc->outbound[i]);
 		if (err) {
 			warning("ua: SIP register failed: %m\n", err);
-			return err;
+			goto out;
 		}
 	}
 
-	return 0;
+ out:
+	mem_deref(reg_uri);
+
+	return err;
 }
 
 
@@ -444,6 +459,9 @@ static void handle_options(struct ua *ua, const struct sip_msg *msg)
 	struct mbuf *desc = NULL;
 	int err;
 
+	debug("ua: incoming OPTIONS message from %r (%J)\n",
+	      &msg->from.auri, &msg->src);
+
 	err = ua_call_alloc(&call, ua, VIDMODE_ON, NULL, NULL, NULL);
 	if (err) {
 		(void)sip_treply(NULL, uag.sip, msg, 500, "Call Error");
@@ -902,8 +920,6 @@ int ua_options_send(struct ua *ua, const char *uri,
 	struct mbuf *dialbuf;
 	int err = 0;
 
-	(void)arg;
-
 	if (!ua || !str_isset(uri))
 		return EINVAL;
 
@@ -917,7 +933,7 @@ int ua_options_send(struct ua *ua, const char *uri,
 
 	dialbuf->buf[dialbuf->end] = '\0';
 
-	err = sip_req_send(ua, "OPTIONS", (char *)dialbuf->buf, resph, NULL,
+	err = sip_req_send(ua, "OPTIONS", (char *)dialbuf->buf, resph, arg,
 			   "Accept: application/sdp\r\n"
 			   "Content-Length: 0\r\n"
 			   "\r\n");
@@ -1142,18 +1158,18 @@ static int add_transp_af(const struct sa *laddr)
 }
 
 
-static int ua_add_transp(void)
+static int ua_add_transp(struct network *net)
 {
 	int err = 0;
 
 	if (!uag.prefer_ipv6) {
-		if (sa_isset(net_laddr_af(AF_INET), SA_ADDR))
-			err |= add_transp_af(net_laddr_af(AF_INET));
+		if (sa_isset(net_laddr_af(net, AF_INET), SA_ADDR))
+			err |= add_transp_af(net_laddr_af(net, AF_INET));
 	}
 
 #if HAVE_INET6
-	if (sa_isset(net_laddr_af(AF_INET6), SA_ADDR))
-		err |= add_transp_af(net_laddr_af(AF_INET6));
+	if (sa_isset(net_laddr_af(net, AF_INET6), SA_ADDR))
+		err |= add_transp_af(net_laddr_af(net, AF_INET6));
 #endif
 
 	return err;
@@ -1247,7 +1263,8 @@ static void net_change_handler(void *arg)
 {
 	(void)arg;
 
-	info("IP-address changed: %j\n", net_laddr_af(AF_INET));
+	info("IP-address changed: %j\n",
+	     net_laddr_af(baresip_network(), AF_INET));
 
 	(void)uag_reset_transp(true, true);
 }
@@ -1307,9 +1324,15 @@ int ua_init(const char *software, bool udp, bool tcp, bool tls,
 	    bool prefer_ipv6)
 {
 	struct config *cfg = conf_config();
+	struct network *net = baresip_network();
 	uint32_t bsize;
 	int err;
 
+	if (!net) {
+		warning("ua: no network\n");
+		return EINVAL;
+	}
+
 	uag.cfg = &cfg->sip;
 	bsize = cfg->sip.trans_bsize;
 
@@ -1319,13 +1342,6 @@ int ua_init(const char *software, bool udp, bool tcp, bool tls,
 	if (err)
 		return err;
 
-	/* Initialise Network */
-	err = net_init(&cfg->net, prefer_ipv6 ? AF_INET6 : AF_INET);
-	if (err) {
-		warning("ua: network init failed: %m\n", err);
-		return err;
-	}
-
 	uag.use_udp = udp;
 	uag.use_tcp = tcp;
 	uag.use_tls = tls;
@@ -1333,14 +1349,14 @@ int ua_init(const char *software, bool udp, bool tcp, bool tls,
 
 	list_init(&uag.ual);
 
-	err = sip_alloc(&uag.sip, net_dnsc(), bsize, bsize, bsize,
+	err = sip_alloc(&uag.sip, net_dnsc(net), bsize, bsize, bsize,
 			software, exit_handler, NULL);
 	if (err) {
 		warning("ua: sip stack failed: %m\n", err);
 		goto out;
 	}
 
-	err = ua_add_transp();
+	err = ua_add_transp(net);
 	if (err)
 		goto out;
 
@@ -1362,7 +1378,7 @@ int ua_init(const char *software, bool udp, bool tcp, bool tls,
 	if (err)
 		goto out;
 
-	net_change(60, net_change_handler, NULL);
+	net_change(net, 60, net_change_handler, NULL);
 
  out:
 	if (err) {
@@ -1379,7 +1395,6 @@ int ua_init(const char *software, bool udp, bool tcp, bool tls,
 void ua_close(void)
 {
 	cmd_unregister(cmdv);
-	net_close();
 	play_close();
 	ui_reset();
 	contact_close();
@@ -1396,6 +1411,9 @@ void ua_close(void)
 
 	list_flush(&uag.ual);
 	list_flush(&uag.ehl);
+
+	/* note: must be done before mod_close() */
+	module_app_unload();
 }
 
 
@@ -1453,6 +1471,20 @@ void ua_stop_all(bool forced)
 }
 
 
+/**
+ * Set the global UA exit handler. The exit handler will be called
+ * asyncronously when the SIP stack has exited.
+ *
+ * @param exith Exit handler
+ * @param arg   Handler argument
+ */
+void uag_set_exit_handler(ua_exit_h *exith, void *arg)
+{
+	uag.exith = exith;
+	uag.arg = arg;
+}
+
+
 /**
  * Reset the SIP transports for all User-Agents
  *
@@ -1463,14 +1495,15 @@ void ua_stop_all(bool forced)
  */
 int uag_reset_transp(bool reg, bool reinvite)
 {
+	struct network *net = baresip_network();
 	struct le *le;
 	int err;
 
 	/* Update SIP transports */
 	sip_transp_flush(uag.sip);
 
-	(void)net_check();
-	err = ua_add_transp();
+	(void)net_check(net);
+	err = ua_add_transp(net);
 	if (err)
 		return err;
 
